/* config.vala for UEvt
 *
 * Copyright (c) 2010 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;

namespace UEvt {

    public class Config : Object {

        private KeyFile keyfile;

        private static const string DEFAULT_CONFIG =
"""[disks]
uevt_udisks=true
uevt_disks_automounting=true
uevt_disks_systray=true
uevt_on_device_added=
uevt_on_device_mounted=
uevt_on_device_removed=
uevt_on_audio_disc=
uevt_on_blank_disc=
uevt_open_blank_disc=
uevt_open_audio_disc=
uevt_open_device=

[power]
uevt_upower=true
uevt_power_systray=true
uevt_power_on_line_power=
uevt_power_on_battery=
uevt_power_on_battery_critical=
uevt_power_battery_critical_lvl=5.0
uevt_power_on_battery_low=
uevt_power_battery_low_lvl=10.0
uevt_power_on_lid_opened=
uevt_power_on_lid_closed=
uevt_power_suspend_on_lid_closed=false
uevt_power_on_sleeping=
uevt_power_on_resuming=
uevt_power_disable_screensaver=xset -dpms;xset s off
uevt_power_enable_screensaver=xset +dpms;xset s on

[cpufreq]
uevt_cpufreq_systray=true
uevt_cpufreq_change_frequency=pkexec uevt_cpu_helper set-global-governor userspace; pkexec uevt_cpu_helper set-global-frequency $2
uevt_cpufreq_change_governor=pkexec uevt_cpu_helper set-global-governor $2

[notify]
uevt_notify_power=true
uevt_notify_disks=true""";


        public static const string UEVT_GROUP_DISKS = "disks";
        public static const string UEVT_GROUP_NOTIFY = "notify";
        public static const string UEVT_GROUP_POWER = "power";
        public static const string UEVT_GROUP_CPUFREQ = "cpufreq";

        public static const string UEVT_UDISKS = "uevt_udisks";
        public static const string UEVT_DISKS_AUTOMOUNTING = "uevt_disks_automounting";
        public static const string UEVT_DISKS_SYSTRAY = "uevt_disks_systray";
        public static const string UEVT_ON_DEVICE_ADDED = "uevt_on_device_added";
        public static const string UEVT_ON_DEVICE_MOUNTED = "uevt_on_device_mounted";
        public static const string UEVT_ON_DEVICE_REMOVED = "uevt_on_device_removed";
        public static const string UEVT_ON_AUDIO_DISC = "uevt_on_audio_disc";
        public static const string UEVT_ON_BLANK_DISC = "uevt_on_blank_disc";
        public static const string UEVT_OPEN_BLANK_DISC = "uevt_open_blank_disc";
        public static const string UEVT_OPEN_AUDIO_DISC = "uevt_open_audio_disc";
        public static const string UEVT_OPEN_DEVICE = "uevt_open_device";

        public static const string UEVT_UPOWER = "uevt_upower";
        public static const string UEVT_POWER_SYSTRAY = "uevt_power_systray";
        public static const string UEVT_POWER_ON_SLEEPING = "uevt_power_on_sleeping";
        public static const string UEVT_POWER_ON_RESUMING = "uevt_power_on_resuming";
        public static const string UEVT_POWER_ON_LID_OPENED = "uevt_power_on_lid_opened";
        public static const string UEVT_POWER_ON_LID_CLOSED = "uevt_power_on_lid_closed";
        public static const string UEVT_POWER_ON_LINE_POWER = "uevt_power_on_line_power";
        public static const string UEVT_POWER_ON_BATTERY = "uevt_power_on_battery";
        public static const string UEVT_POWER_ON_BATTERY_CRITICAL = "uevt_power_on_battery_critical";
        public static const string UEVT_POWER_BATTERY_CRITICAL_LVL = "uevt_power_battery_critical_lvl";
        public static const string UEVT_POWER_ON_BATTERY_LOW = "uevt_power_on_battery_low";
        public static const string UEVT_POWER_BATTERY_LOW_LVL = "uevt_power_battery_low_lvl";
        public static const string UEVT_POWER_SUSPEND_ON_LID_CLOSED = "uevt_power_suspend_on_lid_closed";
        public static const string UEVT_POWER_DISABLE_SCREENSAVER = "uevt_power_disable_screensaver";
        public static const string UEVT_POWER_ENABLE_SCREENSAVER = "uevt_power_enable_screensaver";

        public static const string UEVT_CPUFREQ_SYSTRAY = "uevt_cpufreq_systray";
        public static const string UEVT_CPUFREQ_CHANGE_FREQUENCY = "uevt_cpufreq_change_frequency";
        public static const string UEVT_CPUFREQ_CHANGE_GOVERNOR = "uevt_cpufreq_change_governor";

        public static const string UEVT_NOTIFY_POWER = "uevt_notify_power";
        public static const string UEVT_NOTIFY_DISKS = "uevt_notify_disks";

        construct
        {
            keyfile = new KeyFile();
        }

        public File get_config_file()
        {
            File config_dir = File.new_for_path(
                Environment.get_user_config_dir());
            File uevt_config = config_dir.get_child ("uevt");
            DirUtils.create_with_parents(uevt_config.get_path(), 0777);
            File config_file = uevt_config.get_child ("uevt.conf");

            return config_file;
        }

        public bool load()
        {
            File config_file = this.get_config_file();

            /* if the file doesn't exists create it with default value */
            if (!config_file.query_exists()) {
                FileOutputStream fos = null;

                try {
                    fos = config_file.create(FileCreateFlags.NONE);
                } catch (Error e) {
                    critical("Could not create file %s: %s",
                        config_file.get_path(), e.message);
                    return false;
                }

                /* write the default configuration and close the stream */
                try {
                    fos.write(DEFAULT_CONFIG.data);
                    fos.close();
                } catch (Error e) {
                    critical("Could not write to file %s: %s",
                        config_file.get_path (), e.message);
                    return false;
                }
            }

            try {
                keyfile.load_from_file(config_file.get_path(), 0);
            } catch (KeyFileError e) {
                critical("Could not load configuration: %s", e.message);
                return false;
            } catch (FileError e) {
                critical("Could not load settings: %s", e.message);
                return false;
            }

            return true;
        }

        public bool save()
        {
            File config_file = this.get_config_file();

            FileOutputStream fos = null;
            try {
                fos = config_file.replace(null, true, FileCreateFlags.NONE);
            } catch (Error e) {
                critical("Could not replace file %s: %s",
                    config_file.get_path(), e.message);
                return false;
            }

            string data = null;
            size_t data_len;
            data = this.keyfile.to_data(out data_len);

            try {
                fos.write_all(data.data, null);
                fos.close();
            } catch (Error e) {
                critical("Could not write to file %s: %s",
                    config_file.get_path(), e.message);
                return false;
            }

            return true;
        }

        public string get_string(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_string(group_name, key);
        }

        public bool get_boolean(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_boolean(group_name, key);
        }

        public int get_integer(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_integer(group_name, key);
        }

        public double get_double(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_double(group_name, key);
        }

        public string[] get_string_list(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_string_list(group_name, key);
        }

        public bool[] get_boolean_list(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_boolean_list(group_name, key);
        }

        public int[] get_integer_list(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_integer_list(group_name, key);
        }

        public double[] get_double_list(string group_name, string key) throws KeyFileError
        {
            return this.keyfile.get_double_list(group_name, key);
        }

        public void set_string(string group_name, string key, string val) throws KeyFileError
        {
            this.keyfile.set_string(group_name, key, val);
        }

        public void set_boolean(string group_name, string key, bool val) throws KeyFileError
        {
            this.keyfile.set_boolean(group_name, key, val);
        }

        public void set_double(string group_name, string key, double val) throws KeyFileError
        {
            this.keyfile.set_double(group_name, key, val);
        }

        public void set_integer(string group_name, string key, int val) throws KeyFileError
        {
            this.keyfile.set_integer(group_name, key, val);
        }

        public void set_string_list(string group_name, string key, string[] val) throws KeyFileError
        {
            this.keyfile.set_string_list(group_name, key, val);
        }

        public void set_boolean_list(string group_name, string key, bool[] val) throws KeyFileError
        {
            this.keyfile.set_boolean_list(group_name, key, val);
        }

        public void set_double_list(string group_name, string key, double[] val) throws KeyFileError
        {
            this.keyfile.set_double_list(group_name, key, val);
        }

        public void set_integer_list(string group_name, string key, int[] val) throws KeyFileError
        {
            this.keyfile.set_integer_list(group_name, key, val);
        }
    }
}
