/* configurator.vala for UEvt
 *
 * Copyright (c) 2010 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Gtk;
using ConfigH;

namespace UEvt
{
    public class Configurator : Gtk.Window
    {

        /* General Gtk Widgets */
        private CheckButton plugdisks;
        private CheckButton plugpower;
        private CheckButton notifdisks;
        private CheckButton notifpower;

        /* Disks Gtk Widgets */
        private CheckButton traydisks;
        private CheckButton disksmount;
        private Entry devaddrentry;
        private Entry devmntrentry;
        private Entry devrmrentry;
        private Entry blkdiskentry;
        private Entry addiskentry;
        private Entry opblkdiskentry;
        private Entry opaddiskentry;
        private Entry opdeventry;

        /* Power Gtk Widgets */
        private CheckButton traypower;
        private CheckButton suspendonlid;
        private Entry batentry;
        private Entry linepwrentry;
        private Entry batlowentry;
        private Entry batlowlvlentry;
        private Entry batcrtentry;
        private Entry batcrtlvlentry;
        private Entry resumeentry;
        private Entry sleepentry;
        private Entry lidclosed;
        private Entry lidopened;

        /* Cpu Gtk Widgets */
        private CheckButton traycpu;
        private Entry chgfreq;
        private Entry chggov;

        private void save_configuration()
        {
            try {

            /* Save general configuration */
            config.set_boolean(config.UEVT_GROUP_DISKS,
                               config.UEVT_UDISKS,
                               plugdisks.get_active());
            config.set_boolean(config.UEVT_GROUP_POWER,
                               config.UEVT_UPOWER,
                               plugpower.get_active());

            config.set_boolean(config.UEVT_GROUP_NOTIFY,
                               config.UEVT_NOTIFY_DISKS,
                               notifdisks.get_active());
            config.set_boolean(config.UEVT_GROUP_NOTIFY,
                               config.UEVT_NOTIFY_POWER,
                               notifpower.get_active());

            /* Save disks configuration */
            config.set_boolean(config.UEVT_GROUP_DISKS,
                               config.UEVT_DISKS_SYSTRAY,
                               traydisks.get_active());
            config.set_boolean(config.UEVT_GROUP_DISKS,
                               config.UEVT_DISKS_AUTOMOUNTING,
                               disksmount.get_active());

            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_ON_DEVICE_ADDED,
                              devaddrentry.get_text());
            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_ON_DEVICE_REMOVED,
                              devrmrentry.get_text());
            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_ON_DEVICE_MOUNTED,
                              devmntrentry.get_text());

            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_ON_BLANK_DISC,
                              blkdiskentry.get_text());
            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_ON_AUDIO_DISC,
                              addiskentry.get_text());

            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_OPEN_AUDIO_DISC,
                              opaddiskentry.get_text());
            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_OPEN_BLANK_DISC,
                              opblkdiskentry.get_text());
            config.set_string(config.UEVT_GROUP_DISKS,
                              config.UEVT_OPEN_DEVICE,
                              opdeventry.get_text());

            /* Save power configuration */
            config.set_boolean(config.UEVT_GROUP_POWER,
                               config.UEVT_POWER_SYSTRAY,
                               traypower.get_active());

            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_BATTERY,
                              batentry.get_text());
            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_BATTERY_LOW,
                              batlowentry.get_text());
            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_BATTERY_LOW_LVL,
                              batlowlvlentry.get_text());
            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_BATTERY_CRITICAL,
                              batcrtentry.get_text());
            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_BATTERY_CRITICAL_LVL,
                              batcrtlvlentry.get_text());
            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_LINE_POWER,
                              linepwrentry.get_text());

            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_SLEEPING,
                              sleepentry.get_text());
            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_RESUMING,
                              resumeentry.get_text());

            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_LID_CLOSED,
                              lidclosed.get_text());
            config.set_string(config.UEVT_GROUP_POWER,
                              config.UEVT_POWER_ON_LID_OPENED,
                              lidopened.get_text());
            config.set_boolean(config.UEVT_GROUP_POWER,
                               config.UEVT_POWER_SUSPEND_ON_LID_CLOSED,
                               suspendonlid.get_active());

            /* Save CPU configuration */
            config.set_boolean(config.UEVT_GROUP_CPUFREQ,
                               config.UEVT_CPUFREQ_SYSTRAY,
                               traycpu.get_active());
            config.set_string(config.UEVT_GROUP_CPUFREQ,
                              config.UEVT_CPUFREQ_CHANGE_FREQUENCY,
                              chgfreq.get_text());
            config.set_string(config.UEVT_GROUP_CPUFREQ,
                              config.UEVT_CPUFREQ_CHANGE_GOVERNOR,
                              chggov.get_text());

            } catch(KeyFileError kf) {
                critical("Error when writing UEvt configuration : %s", kf.message);
            }

            config.save();
        }


        private Widget get_general_widget()
        {
            var vbox = new VBox(false, 10);

            var label = new Label("");
            label.set_markup(_("<b>Some changes can require a restart of the program to work !</b>"));
            vbox.pack_start(label, false, false, 5);

            /* Plugins part components*/
            var frameplug = new Frame(_("Plugins"));
            var vboxplug = new VBox(true, 5);
            plugdisks = new CheckButton.with_label(_("Enable disks plugin"));
            plugpower = new CheckButton.with_label(_("Enable power plugin"));
            vboxplug.add(plugdisks);
            vboxplug.add(plugpower);
            frameplug.add(vboxplug);

            var framenotif = new Frame(_("Notifications"));
            var vboxnotif = new VBox(true, 5);
            notifdisks = new CheckButton.with_label(_("Enable disks notifications"));
            notifpower = new CheckButton.with_label(_("Enable power notifications"));
            vboxnotif.add(notifdisks);
            vboxnotif.add(notifpower);
            framenotif.add(vboxnotif);

            vbox.pack_start(frameplug, false, false, 5);
            vbox.pack_start(framenotif, false, false, 5);

            /* Initialize components with proper configuration */
            try {
                plugdisks.set_active(config.get_boolean(config.UEVT_GROUP_DISKS,
                                                        config.UEVT_UDISKS));
                plugpower.set_active(config.get_boolean(config.UEVT_GROUP_POWER,
                                                        config.UEVT_UPOWER));
                notifdisks.set_active(config.get_boolean(config.UEVT_GROUP_NOTIFY,
                                                    config.UEVT_NOTIFY_DISKS));
                notifpower.set_active(config.get_boolean(config.UEVT_GROUP_NOTIFY,
                                                    config.UEVT_NOTIFY_POWER));
            } catch(KeyFileError kf) {
                critical("Error when reading UEvt configuration : %s", kf.message);
            }

            return vbox;
        }

        private Widget get_disks_widget()
        {
            var vbox = new VBox(false, 10);
            Label label;

            /* Plugins part components*/
            traydisks = new CheckButton.with_label(_("Enable disks systray"));
            disksmount = new CheckButton.with_label(_("Enable disks automounting"));
            label = new Label("");
            label.set_markup(_("<span style=\"italic\"><b>WARNING</b> : The above option need a polkit authorization to work. See the documentation.</span>"));
            vbox.pack_start(traydisks, false, false, 5);
            vbox.pack_start(disksmount, false, false, 5);
            vbox.pack_start(label, false, false, 0);


            /* Devices frame */
            var frame = new Frame(_("Devices Commands"));
            var vbox2 = new VBox(false, 5);

            label = new Label("");
            label.set_markup(_("<span style=\"italic\"><b>You can use the following parameters in the below commands :</b>\nParameter ‘$1′ : Device’s path (ex: /dev/sdb).\nParameter ‘$2′ : Mount path (ex: /media/xxxx-xxxx/) - Only with \"On Device Mounted\".</span>"));
            vbox2.add(label);

            var hbox = new HBox(true, 5);
            label = new Label(_("On Device Added :"));
            devaddrentry = new Entry();
            hbox.add(label);
            hbox.add(devaddrentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Device Mounted :"));
            devmntrentry = new Entry();
            hbox.add(label);
            hbox.add(devmntrentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Device Removed :"));
            devrmrentry = new Entry();
            hbox.add(label);
            hbox.add(devrmrentry);
            vbox2.add(hbox);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);


            /* Optical disks frame */
            frame = new Frame(_("Optical Disks Commands"));
            vbox2 = new VBox(false, 10);

            label = new Label("");
            label.set_markup(_("<span style=\"italic\"><b>\nYou can use the following parameters in the below commands :</b>\nParameter ‘$1′ : Device’s path (ex: /dev/sdc).</span>"));
            vbox2.add(label);

            hbox = new HBox(true, 5);
            label = new Label(_("On Blank Disk :"));
            blkdiskentry = new Entry();
            hbox.add(label);
            hbox.add(blkdiskentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Audio Disk :"));
            addiskentry = new Entry();
            hbox.add(label);
            hbox.add(addiskentry);
            vbox2.add(hbox);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);

            /* Opening commands frame */
            frame = new Frame(_("Opening Commands (for the systray)"));
            vbox2 = new VBox(false, 10);

            label = new Label("");
            label.set_markup(_("<span style=\"italic\"><b>\nYou can use the following parameters in the below commands :</b>\nParameter ‘$1′ : Device’s path (ex: /dev/sdb).\nParameter ‘$2′ : Mount path (ex: /media/xxxx-xxxx/) - Only with \"Open Other Device\".</span>"));
            vbox2.add(label);

            hbox = new HBox(true, 5);
            label = new Label(_("Open Blank Disk :"));
            opblkdiskentry = new Entry();
            hbox.add(label);
            hbox.add(opblkdiskentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("Open Audio Disk :"));
            opaddiskentry = new Entry();
            hbox.add(label);
            hbox.add(opaddiskentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("Open Other Device :"));
            opdeventry = new Entry();
            hbox.add(label);
            hbox.add(opdeventry);
            vbox2.add(hbox);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);

            /* Initialize components with proper configuration */
            try {
                traydisks.set_active(config.get_boolean(config.UEVT_GROUP_DISKS,
                                                config.UEVT_DISKS_SYSTRAY));
                disksmount.set_active(config.get_boolean(config.UEVT_GROUP_DISKS,
                                            config.UEVT_DISKS_AUTOMOUNTING));

                devaddrentry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_ON_DEVICE_ADDED));
                devrmrentry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_ON_DEVICE_REMOVED));
                devmntrentry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_ON_DEVICE_MOUNTED));

                blkdiskentry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_ON_BLANK_DISC));
                addiskentry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_ON_AUDIO_DISC));

                opaddiskentry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_OPEN_AUDIO_DISC));
                opblkdiskentry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_OPEN_BLANK_DISC));
                opdeventry.set_text(config.get_string(config.UEVT_GROUP_DISKS,
                                                config.UEVT_OPEN_DEVICE));
            } catch(KeyFileError kf) {
                critical("Error when reading UEvt configuration : %s", kf.message);
            }

            return vbox;
        }

        private Widget get_power_widget()
        {
            var vbox = new VBox(false, 15);
            Label label;

            traypower = new CheckButton.with_label(_("Enable power systray"));
            vbox.pack_start(traypower, false, false, 5);

            /* Devices frame */
            var frame = new Frame(_("Power Events"));
            var vbox2 = new VBox(true, 10);

            var hbox = new HBox(true, 5);
            label = new Label(_("On Battery :"));
            batentry = new Entry();
            hbox.add(label);
            hbox.add(batentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Battery Low :"));
            batlowentry = new Entry();
            hbox.add(label);
            hbox.add(batlowentry);
            vbox2.add(hbox);

             hbox = new HBox(true, 5);
            label = new Label(_("Battery Low Level (in %) :"));
            batlowlvlentry = new Entry();
            hbox.add(label);
            hbox.add(batlowlvlentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Battery Critical :"));
            batcrtentry = new Entry();
            hbox.add(label);
            hbox.add(batcrtentry);
            vbox2.add(hbox);

             hbox = new HBox(true, 5);
            label = new Label(_("Battery Critical Level (in %) :"));
            batcrtlvlentry = new Entry();
            hbox.add(label);
            hbox.add(batcrtlvlentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Line Power :"));
            linepwrentry = new Entry();
            hbox.add(label);
            hbox.add(linepwrentry);
            vbox2.add(hbox);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);


            /* Sleeping frame */
            frame = new Frame(_("Suspend Events"));
            vbox2 = new VBox(true, 10);

            hbox = new HBox(true, 5);
            label = new Label(_("On Sleeping :"));
            sleepentry = new Entry();
            hbox.add(label);
            hbox.add(sleepentry);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Resuming :"));
            resumeentry = new Entry();
            hbox.add(label);
            hbox.add(resumeentry);
            vbox2.add(hbox);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);

            /* Opening commands frame */
            frame = new Frame(_("Lid Events (laptop only)"));
            vbox2 = new VBox(false, 10);

            hbox = new HBox(true, 5);
            label = new Label(_("On Lid Closed :"));
            lidclosed = new Entry();
            hbox.add(label);
            hbox.add(lidclosed);
            vbox2.add(hbox);

            hbox = new HBox(true, 5);
            label = new Label(_("On Lid Opened :"));
            lidopened = new Entry();
            hbox.add(label);
            hbox.add(lidopened);
            vbox2.add(hbox);

            suspendonlid = new CheckButton.with_label(_("Suspend the computer when the lid is closed"));
            vbox2.add(suspendonlid);

            label = new Label("");
            label.set_markup(_("<span style=\"italic\"><b>WARNING</b> : The above option need a polkit authorization to work. See the documentation.</span>"));
            vbox2.add(label);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);

            /* Initialize components with proper configuration */
            try {
                traypower.set_active(config.get_boolean(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_SYSTRAY));

                batentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_BATTERY));
                batlowentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_BATTERY_LOW));
                batlowlvlentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_BATTERY_LOW_LVL));
                batcrtentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_BATTERY_CRITICAL));
                batcrtlvlentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_BATTERY_CRITICAL_LVL));
                linepwrentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_LINE_POWER));

                sleepentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_SLEEPING));
                resumeentry.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_RESUMING));

                lidclosed.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_LID_CLOSED));
                lidopened.set_text(config.get_string(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_ON_LID_OPENED));
                suspendonlid.set_active(config.get_boolean(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_SUSPEND_ON_LID_CLOSED));

            } catch(KeyFileError kf) {
                critical("Error when reading UEvt configuration : %s", kf.message);
            }

            return vbox;
        }

        private Widget get_cpu_widget()
        {
            var vbox = new VBox(false, 5);
            Label label;

            traycpu = new CheckButton.with_label(_("Enable cpu systray"));
            vbox.pack_start(traycpu, false, false, 5);

            var frame = new Frame(_("CPU Managing Commands"));
            var vbox2 = new VBox(false, 10);

            label = new Label("");
            label.set_markup(_("<span style=\"italic\"><b>\nYou can use the following parameters in the below commands :</b>\nParameter ‘$1′ : CPU number (always 0 for now).\nParameter ‘$2′ : Frequency (ex: 800000) in first command\nGovernor name (ex: powersave) in the second one.</span>"));
            vbox2.add(label);

            var hbox = new HBox(true, 5);
            label = new Label(_("Change CPU Frequency :"));
            chgfreq = new Entry();
            hbox.add(label);
            hbox.add(chgfreq);
            vbox2.pack_start(hbox, false, false, 5);

            hbox = new HBox(true, 5);
            label = new Label(_("Change CPU Governor :"));
            chggov = new Entry();
            hbox.add(label);
            hbox.add(chggov);
            vbox2.pack_start(hbox, false, false, 5);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);

            /* Initialize components with proper configuration */
            try {
                traycpu.set_active(config.get_boolean(config.UEVT_GROUP_CPUFREQ,
                                                        config.UEVT_CPUFREQ_SYSTRAY));

                chgfreq.set_text(config.get_string(config.UEVT_GROUP_CPUFREQ,
                                                config.UEVT_CPUFREQ_CHANGE_FREQUENCY));
                chggov.set_text(config.get_string(config.UEVT_GROUP_CPUFREQ,
                                                config.UEVT_CPUFREQ_CHANGE_GOVERNOR));
            } catch(KeyFileError kf) {
                critical("Error when reading UEvt configuration : %s", kf.message);
            }

            return vbox;
        }
        public Configurator()
        {
            this.title = _("UEvt Configuration");
            this.position = WindowPosition.CENTER;
            this.set_icon_name("drive-removable-media-2");

            var vbox = new VBox(false, 5);

            var notebook = new Notebook();
            notebook.append_page(get_general_widget(), new Label(_("General")));
            notebook.append_page(get_disks_widget(), new Label(_("Disks Configuration")));
            notebook.append_page(get_power_widget(), new Label(_("Power Configuration")));
            notebook.append_page(get_cpu_widget(), new Label(_("Cpu Configuration")));
            vbox.add(notebook);

            vbox.add(new HSeparator());

            var hbox = new HBox(true, 5);
            var apply = new Button.from_stock(Gtk.Stock.APPLY);
            apply.clicked.connect (() => {
                save_configuration();
                this.destroy();
            });

            var cancel = new Button.from_stock(Gtk.Stock.CANCEL);
            cancel.clicked.connect (() => {
                this.destroy();
            });

            hbox.pack_start(apply, false, false, 5);
            hbox.pack_start(cancel, false, false, 5);
            vbox.pack_start(hbox, false, false, 5);

            this.add(vbox);
        }
    }
}
