/* cpu-systray.vala for UEvt
 *
 * Copyright (c) 2010,2011 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Gtk;
using ConfigH;
using UEvt;

namespace UCpu
{

    public class CpuTray : Gtk.StatusIcon
    {
        public int cpu_number;

        private void detect_cpu_number()
        {
            int i = 0;
            GLib.File file;

            while(true) {
                file = File.new_for_path("/sys/devices/system/cpu/cpu%d/cpufreq/scaling_cur_freq"
                                        .printf(i));
                if (!file.query_exists())
                    break;
                else
                    i++;
            }

            cpu_number = i;
        }

        private string get_cpu_max_freq(int cpu_num)
        {
            string freq = "";

            if(cpu_num >= cpu_number)
                return "";

            GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%d/cpufreq/scaling_max_freq"
                                     .printf(cpu_num));

            try {
                var dis = new DataInputStream (file.read());
                freq = dis.read_line(null);
            } catch (Error e) {
                message("%s", e.message);
            }

            return freq;
        }

        private string get_cpu_min_freq(int cpu_num)
        {
            string freq = "";

            if(cpu_num >= cpu_number)
                return "";

            GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%d/cpufreq/scaling_min_freq"
                                     .printf(cpu_num));

            try {
                var dis = new DataInputStream (file.read());
                freq = dis.read_line(null);
            } catch (Error e) {
                message("%s", e.message);
            }

            return freq;
        }

        private string get_cpu_freq(int cpu_num)
        {
            string freq = "";

            if(cpu_num >= cpu_number)
                return "";

            GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%d/cpufreq/scaling_cur_freq"
                                     .printf(cpu_num));

            try {
                var dis = new DataInputStream (file.read());
                freq = dis.read_line(null);
            } catch (Error e) {
                message("%s", e.message);
            }

            return freq;
        }

        private string[] get_available_cpu_freqs(int cpu_num)
        {
            string[] freqs = {};

            if(cpu_num >= cpu_number)
                return {};

            GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%d/cpufreq/scaling_available_frequencies"
                                     .printf(cpu_num));

            try {
                var dis = new DataInputStream (file.read());
                string line = dis.read_line(null);
                if(line != null)
                    freqs = line.split(" ", 0);
            } catch (Error e) {
                message("%s", e.message);
            }

            return freqs;
        }

        private string[] get_available_cpu_govs(int cpu_num)
        {
            string[] govs = {};

            if(cpu_num >= cpu_number)
                return {};

            GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%d/cpufreq/scaling_available_governors"
                                     .printf(cpu_num));

            try {
                var dis = new DataInputStream (file.read());
                string line = dis.read_line(null);
                if(line != null)
                    govs = line.split(" ", 0);
            } catch (Error e) {
                message("%s", e.message);
            }

            return govs;
        }

        private string get_cpu_gov(int cpu_num)
        {
            string gov = "";

            if(cpu_num >= cpu_number)
                return "";

            GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor"
                                     .printf(cpu_num));

            try {
                var dis = new DataInputStream (file.read());
                gov = dis.read_line(null);
            } catch (Error e) {
                message("%s", e.message);
            }

            return gov;
        }

        private void update_cpu_icon()
        {
            uint64 cur_freq = uint64.parse(get_cpu_freq(0));
            uint64 max_freq = uint64.parse(get_cpu_max_freq(0));
            uint64 min_freq = uint64.parse(get_cpu_min_freq(0));

            if(max_freq == 0) {
                this.set_from_gicon(new ThemedIcon("cpufreq-na"));
                return;
            }

            float ratio = ((float)(cur_freq - min_freq) / (max_freq - min_freq)) * 100;
            if(ratio <= 25.0)
                this.set_from_gicon(new ThemedIcon("cpufreq-25"));
            else if(ratio <= 50.0)
                this.set_from_gicon(new ThemedIcon("cpufreq-50"));
            else if(ratio <= 75.0)
                this.set_from_gicon(new ThemedIcon("cpufreq-75"));
            else
                this.set_from_gicon(new ThemedIcon("cpufreq-100"));
        }

        public CpuTray()
        {
            this.set_tooltip_text("UEvt CpuFreq");
            this.visible = true;
            this.popup_menu.connect(popup_menu_callback);

            detect_cpu_number();
            update_cpu_icon();
        }

        private void about_clicked()
        {
            string authors[2];
            authors[0] = "Boris Albar";
            authors[1] = null;

            var about = new AboutDialog();
            about.set_version(PACKAGE_VERSION);
            about.set_program_name("UEvt CpuFreq");
            about.set_license(licence);
            about.set_authors(authors);
            Image img = new Image.from_file(ICONDATADIR + "status/cpufreq-na.png");
            about.set_logo(img.get_pixbuf());
            about.set_comments("A lightweight, desktop independant software for managing CPU.");
            about.set_copyright("(C) 2010 Boris Albar <elentir@frugalware.org>");
            about.set_website("http://elentir.sleipnir.fr");
            about.run();
            about.hide();
        }

        private void popup_menu_callback(uint button, uint activate_time)
        {
            Gtk.Menu mainmenu = new Gtk.Menu();
            RadioMenuItem radio = null;
            string cur_gov = get_cpu_gov(0);
            string cur_freq = get_cpu_freq(0);

            /* Frequencies menu items */
            foreach (string elem in get_available_cpu_freqs(0)) {
                if(elem != "") {
                    string ghz = "%.2f".printf(((float)uint64.parse(elem))/1000000);
                    radio = new RadioMenuItem.with_label(radio == null ?
                                                null : radio.get_group(),
                                                ghz + " GHz");
                    radio.set_data<string>("frequency", elem);

                    if(cur_freq == elem)
                        radio.set_active(true);

                    radio.toggled.connect((source) => {
                        if(source.get_active() == true) {
                            run_command_config(config.UEVT_GROUP_CPUFREQ,
                                    config.UEVT_CPUFREQ_CHANGE_FREQUENCY,
                                    "0", source.get_data<string>("frequency"),
                                    true);

                            update_cpu_icon();
                        }
                    });

                    mainmenu.append(radio);
                }
            }
            radio = null;

            SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();
            mainmenu.append(separator);

            /* Governors menu items */
            foreach (string elem in get_available_cpu_govs(0)) {
                if(elem != "") {
                    radio = new RadioMenuItem.with_label(radio == null ?
                                                null : radio.get_group(),
                                                elem);
                    if(cur_gov == elem)
                        radio.set_active(true);

                    radio.toggled.connect((source) => {
                        if(source.get_active() == true) {
                            run_command_config(config.UEVT_GROUP_CPUFREQ,
                                    config.UEVT_CPUFREQ_CHANGE_GOVERNOR,
                                    "0", source.get_label(), true);
                            update_cpu_icon();
                        }
                    });

                    mainmenu.append(radio);
                }
            }

            separator = new Gtk.SeparatorMenuItem();
            mainmenu.append(separator);

            ImageMenuItem menuConfig = new ImageMenuItem.from_stock(Gtk.Stock.PREFERENCES, null);
            menuConfig.activate.connect((source) => {
                Configurator configurator = new Configurator();
                configurator.show_all();
            });
            mainmenu.append(menuConfig);

            ImageMenuItem menuAbout = new ImageMenuItem.from_stock(Gtk.Stock.ABOUT, null);
            menuAbout.activate.connect(about_clicked);
            mainmenu.append(menuAbout);

            ImageMenuItem menuExit = new ImageMenuItem.from_stock(Gtk.Stock.QUIT, null);
            menuExit.activate.connect((source) => {
                if(UEvt.powertray != null || UEvt.diskstray != null) {
                    this.visible = false;
                    UEvt.cputray = null;
                } else {
                    UEvt.on_exit(0);
                }});
            mainmenu.append(menuExit);

            mainmenu.show_all();
            mainmenu.popup(null,null, null, button, activate_time);
        }
    }
}