/* uevt-cpu-helper.vala for UEvt
 *
 * Copyright (c) 2011 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Posix;

namespace UEvtCpuHelper
{
    string[] availcpulist;

    private string[] uevt_cpu_helper_detect_cpu_number()
    {
        string prescpu = "";
        GLib.File file = File.new_for_path("/sys/devices/system/cpu/present");

        try {
            var dis = new DataInputStream (file.read());
            prescpu = dis.read_line(null);
        } catch (Error e) {
            message("%s", e.message);
            return {};
        }

        string[] availcpu = prescpu.split("-", 0);

        return availcpu;
    }

    private string uevt_cpu_helper_get_cpu_max_freq(string cpu_num)
    {
        string freq = "";

        if(!(cpu_num in availcpulist))
            return "";

        GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_max_freq"
                                     .printf(cpu_num));

        try {
            var dis = new DataInputStream (file.read());
            freq = dis.read_line(null);
        } catch (Error e) {
            message("%s", e.message);
        }

        return freq;
    }

    private string uevt_cpu_helper_get_cpu_min_freq(string cpu_num)
    {
        string freq = "";

        if(!(cpu_num in availcpulist))
            return "";

        GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_min_freq"
                                     .printf(cpu_num));

        try {
            var dis = new DataInputStream (file.read());
            freq = dis.read_line(null);
        } catch (Error e) {
            message("%s", e.message);
        }

        return freq;
    }

    private string uevt_cpu_helper_get_frequency(string cpu_num)
    {
        string freq = "";

        if(!(cpu_num in availcpulist))
            return "";

        GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_cur_freq"
                                     .printf(cpu_num));

        try {
            var dis = new DataInputStream (file.read());
            freq = dis.read_line(null);
        } catch (Error e) {
            message("%s", e.message);
        }

        return freq;
    }

    private void uevt_cpu_helper_set_frequency(string cpu_num, string newfreq)
    {
        string newfreqstring = newfreq.to_string();

        if(!(newfreqstring in uevt_cpu_helper_get_available_cpu_freqs(cpu_num))) {
            Posix.stdout.printf("Unrecognized frequency %s\n", newfreq);
            return;
        }

        FILE file = FILE.open("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_setspeed"
                                  .printf(cpu_num), "w");
        file.puts(newfreq);

        return;
    }

    private void uevt_cpu_helper_set_global_frequency(string newfrequency)
    {
        foreach(string cpu in availcpulist)
            uevt_cpu_helper_set_frequency(cpu, newfrequency);

        return;
    }

    private string[] uevt_cpu_helper_get_available_cpu_freqs(string cpu_num)
    {
        string[] freqs = {};

        if(!(cpu_num in availcpulist))
            return {};

        GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_available_frequencies"
                                     .printf(cpu_num));

        try {
            var dis = new DataInputStream (file.read());
            string line = dis.read_line(null);
            if(line != null)
                freqs = line.split(" ", 0);
        } catch (Error e) {
            message("%s", e.message);
        }

        return freqs;
    }

    private string[] uevt_cpu_helper_get_available_governors(string cpu_num)
    {
        string[] govs = {};

        if(!(cpu_num in availcpulist))
            return {};

        GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_available_governors"
                                     .printf(cpu_num));

        try {
            var dis = new DataInputStream (file.read());
            string line = dis.read_line(null);
            if(line != null)
                govs = line.split(" ", 0);
        } catch (Error e) {
            message("%s", e.message);
        }

        return govs;
    }

    private string uevt_cpu_helper_get_governor(string cpu_num)
    {
        string gov = "";

        if(!(cpu_num in availcpulist))
            return "";

        GLib.File file = File.new_for_path("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_governor"
                                           .printf(cpu_num));

        try {
            var dis = new DataInputStream (file.read());
            gov = dis.read_line(null);
        } catch (Error e) {
            message("%s", e.message);
        }

        return gov;
    }

    private void uevt_cpu_helper_set_governor(string cpu_num, string newgovernor)
    {
        if(!(newgovernor in uevt_cpu_helper_get_available_governors(cpu_num))) {
            Posix.stdout.printf("Unrecognized governor %s\n", newgovernor);
            return;
        }

        FILE file = FILE.open("/sys/devices/system/cpu/cpu%s/cpufreq/scaling_governor"
                              .printf(cpu_num), "w");
        file.puts(newgovernor);

        return;
    }

    private void uevt_cpu_helper_set_global_governor(string newgovernor)
    {
        foreach(string cpu in availcpulist)
            uevt_cpu_helper_set_governor(cpu, newgovernor);

        return;
    }

    private void uevt_cpu_helper_set_global_min_frequency()
    {
        string minfreq;

        foreach(string cpu in availcpulist) {
            minfreq = uevt_cpu_helper_get_cpu_min_freq(cpu);
            uevt_cpu_helper_set_frequency(cpu, minfreq);
        }

        return;
    }

    private void uevt_cpu_helper_set_global_max_frequency()
    {
        string maxfreq;

        foreach(string cpu in availcpulist) {
            maxfreq = uevt_cpu_helper_get_cpu_max_freq(cpu);
            uevt_cpu_helper_set_frequency(cpu, maxfreq);
        }

        return;
    }

    public static int main (string[] args)
    {
        if(Posix.getuid() != 0) {
            Posix.stdout.printf("This program must be run with root privileges.\n");
            return 1;
        }

        availcpulist = uevt_cpu_helper_detect_cpu_number();

        if(args[1] == "show-infos") {
            foreach(string cpu in availcpulist) {
                Posix.stdout.printf("CPU %s : current governor %s, current frequency %s\n",
                                    cpu,
                                    uevt_cpu_helper_get_governor(cpu),
                                    uevt_cpu_helper_get_frequency(cpu));
            }
        } else if(args[1] == "set-global-governor") {
                uevt_cpu_helper_set_global_governor(args[2]);
        } else if(args[1] == "set-global-frequency") {
                uevt_cpu_helper_set_global_frequency(args[2]);
        } else if(args[1] == "set-global-min-frequency") {
                uevt_cpu_helper_set_global_min_frequency();
        } else if(args[1] == "set-global-max-frequency") {
                uevt_cpu_helper_set_global_max_frequency();
        } else {
            Posix.stdout.printf("Unrecognized option %s\n", args[1]);
            Posix.stdout.printf("Usage : uevt_cpu_helper [OPERATION] [ARGUMENT] \n\n");
            Posix.stdout.printf("    show-infos : Show current governor and frequency for all cpus\n");
            Posix.stdout.printf("    set-global-governor : set governor of all cpus (Argument : governor name)\n");
            Posix.stdout.printf("    set-global-frequency : set frequency of all cpus (Argument : frequency)\n");
            Posix.stdout.printf("    set-global-min-frequency : set frequency of all cpus to minimum\n");
            Posix.stdout.printf("    set-global-max-frequency : set frequency of all cpus to maximum\n");
        }

        return 0;
    }
}
