/* logger.vala for UEvt
 *
 * Copyright (c) 2010 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Logger {

    FileOutputStream file_stream;

    File get_log_file()
    {
        File config_dir = File.new_for_path (
            Environment.get_user_config_dir());
        File uevt_log = config_dir.get_child ("uevt");
        DirUtils.create_with_parents(uevt_log.get_path(), 0777);
        File log_file = uevt_log.get_child ("uevt.log");

        return log_file;
    }

    bool Init(string filename) {
        Log.set_handler("", LogLevelFlags.LEVEL_INFO, info_handler);
        Log.set_handler("", LogLevelFlags.LEVEL_DEBUG, debug_handler);
        Log.set_handler("", LogLevelFlags.LEVEL_MESSAGE, message_handler);
        Log.set_handler("", LogLevelFlags.LEVEL_WARNING, warning_handler);
        Log.set_handler("", LogLevelFlags.LEVEL_CRITICAL, critical_handler);
        Log.set_fatal_mask("", LogLevelFlags.LEVEL_CRITICAL);

        try {
            File file = get_log_file();
            if (file.query_exists())
                file.delete(null);
            file_stream = file.create(FileCreateFlags.NONE, null);

            if (!file.query_exists())
                return false;

            file_stream.write("Logging File Open\n".data);
        } catch (Error e) {
            stderr.printf("%s", e.message);
            return false;
        }
        return true;
    }

    void Close() {
        try {
            file_stream.write("Logging File Closed\n".data);
            file_stream.flush();
            file_stream.close();
        } catch(Error e) {
            stderr.printf("%s\n", e.message);
        }
    }

    void log(FileOutputStream stream, string prefix, string message) {
        try {
            stream.write(prefix.data);
            stream.write(message.data);
            stream.write("\n".data);
            stream.flush();
        } catch(Error e) {
            stderr.printf("%s\n", e.message);
        }
    }

    void info_handler(string? domain, LogLevelFlags flags, string message) {
        log(file_stream, "[INF] ", message);
    }

    void debug_handler(string? domain, LogLevelFlags flags, string message) {
        log(file_stream, "[DBG] ", message);
    }

    void message_handler(string? domain, LogLevelFlags flags, string message) {
        log(file_stream, "[MSG] ", message);
    }

    void warning_handler(string? domain, LogLevelFlags flags, string message) {
        log(file_stream, "[WRN] ", message);
    }

    void critical_handler(string? domain, LogLevelFlags flags, string message) {
        log(file_stream, "[CRT] ", message);
    }
}

