/* upower-infos.vala for UEvt
 *
 * Copyright (c) 2010,2011 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Gtk;

namespace UPower
{
    public class PowerInfos : Gtk.Window
    {
        public PowerInfos(string vendor, string model, string serial, uint type, uint tech,
                        double capacity, double energy, double energyempty, double energyfull,
                        double energyfulldesign, double energyrate, double voltage)
        {
            this.title = _("Power Device Informations");
            this.position = WindowPosition.CENTER;
            this.set_icon_name("battery-075");

            var vbox = new VBox(false, 5);

            var frame = new Frame(_("General Properties"));
            var vbox2 = new VBox(true, 10);

            Label vendorlabel;
            Label modellabel;
            Label seriallabel;
            Label techlabel;
            Label typelabel;
            Label capacitylabel;

            if(vendor == "")
                vendorlabel = new Label(_("Vendor : Unknown"));
            else
                vendorlabel = new Label(_("Vendor : ") + vendor);

            if(model == "")
                modellabel = new Label(_("Model : Unknown"));
            else
                modellabel = new Label(_("Model : ") + model);

            if(serial == "")
                seriallabel = new Label(_("Serial : Unknown"));
            else
                seriallabel = new Label(_("Serial : ") + serial);

            if(type == 1)
                typelabel = new Label(_("Type : Line Power"));
            else if(type == 2)
                typelabel = new Label(_("Type : Battery"));
            else if(type == 3)
                typelabel = new Label(_("Type : UPS"));
            else if(type == 4)
                typelabel = new Label(_("Type : Monitor"));
            else if(type == 5)
                typelabel = new Label(_("Type : Mouse battery"));
            else if(type == 6)
                typelabel = new Label(_("Type : Keyboard battery"));
            else if(type == 7)
                typelabel = new Label(_("Type : Pda"));
            else if(type == 8)
                typelabel = new Label(_("Type : Phone"));
            else
                typelabel = new Label(_("Type : Unknown"));

            if(type == 2) {
                if(tech == 1)
                    techlabel = new Label(_("Technology : Lithium ion"));
                else if(tech == 2)
                    techlabel = new Label(_("Technology : Lithium polymer"));
                else if(tech == 3)
                    techlabel = new Label(_("Technology : Lithium iron phosphate"));
                else if(tech == 4)
                    techlabel = new Label(_("Technology : Lead acid"));
                else if(tech == 5)
                    techlabel = new Label(_("Technology : Nickel cadmium"));
                else if(tech == 6)
                    techlabel = new Label(_("Technology : Nickel metal hydride"));
                else
                    techlabel = new Label(_("Technology : Unknown"));
            } else {
                techlabel = new Label(_("Technology : Not Available"));
            }

            if(type == 2) {
                capacitylabel = new Label(_("Capacity : ") + capacity.to_string() + " %");
            } else {
                capacitylabel = new Label(_("Capacity : Not available"));
            }

            vbox2.add(vendorlabel);
            vbox2.add(modellabel);
            vbox2.add(seriallabel);
            vbox2.add(typelabel);
            vbox2.add(techlabel);
            vbox2.add(capacitylabel);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);

            frame = new Frame(_("Energy"));
            vbox2 = new VBox(true, 10);

            Label energylabel;
            Label energyemptylabel;
            Label energyfulllabel;
            Label energyfulldlabel;
            Label energyratelabel;
            Label voltagelabel;

            energylabel = new Label(_("Energy : ") + "%.3f".printf(energy) + " Wh");
            energyemptylabel = new Label(_("Energy (empty) : ") + "%.3f".printf(energyempty) + " Wh");
            energyfulllabel = new Label(_("Energy (full) : ") + "%.3f".printf(energyfull) + " Wh");
            energyfulldlabel = new Label(_("Energy (full design) : ") + "%.3f".printf(energyfulldesign) + " Wh");
            energyratelabel = new Label(_("Energy rate : ") + "%.3f".printf(energyrate) + " Wh");
            voltagelabel = new Label(_("Voltage : ") + "%.3f".printf(voltage) + " Wh");

            vbox2.add(energylabel);
            vbox2.add(energyemptylabel);
            vbox2.add(energyfulllabel);
            vbox2.add(energyfulldlabel);
            vbox2.add(energyratelabel);
            vbox2.add(voltagelabel);

            frame.add(vbox2);
            vbox.pack_start(frame, false, false, 5);

            var okbutton = new Button.from_stock(Gtk.Stock.OK);
            okbutton.clicked.connect (() => {
                this.destroy();
            });

            vbox.pack_start(okbutton, false, false, 5);

            this.add(vbox);
        }
    }
}