/* uevt-backlight.vala for UEvt
 *
 * Copyright (c) 2011 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace UPower
{

    /* available kernel interfaces in priority order */
    static const string[] backlight_interfaces = {
        "nv_backlight",
        "asus_laptop",
        "toshiba",
        "eeepc",
        "thinkpad_screen",
        "acpi_video1",
        "mbp_backlight",
        "acpi_video0",
        "fujitsu-laptop",
        "sony",
        "samsung"
    };

    private string uevt_backlight_get_best_backlight()
    {
        foreach (string blif in backlight_interfaces) {
            if(FileUtils.test("/sys/class/backlight/%s".printf(blif),
                                                FileTest.EXISTS) &&
                FileUtils.test("/sys/class/backlight/%s".printf(blif),
                                                    FileTest.IS_DIR))
                    return blif;
        }

        return "";
    }

    private int uevt_backlight_get_max_brightness()
    {
        string maxbl = "";
        string blif = uevt_backlight_get_best_backlight();

        if(blif == "")
            return -1;

        File file = File.new_for_path("/sys/class/backlight/%s/max_brightness"
                                     .printf(blif));

        try {
            var dis = new DataInputStream(file.read());
            maxbl = dis.read_line(null);
        } catch (Error e) {
            return -2;
        }

        return int.parse(maxbl);
    }

    private int uevt_backlight_get_brightness()
    {
        string curbl = "";
        string blif = uevt_backlight_get_best_backlight();

        if(blif == "")
            return -1;

        File file = File.new_for_path("/sys/class/backlight/%s/actual_brightness"
                                     .printf(blif));

        try {
            var dis = new DataInputStream(file.read());
            curbl = dis.read_line(null);
        } catch (Error e) {
            return -2;
        }

        return int.parse(curbl);
    }
}