/* uevt.vala for UEvt
 *
 * Copyright (c) 2010 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using ConfigH;

static const string licence=
"""This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.""";

namespace UEvt
{
    private static MainLoop mainloop;
    public Config config;
    public UDisks.DisksTray diskstray;
    public UPower.PowerTray powertray;
    public UCpu.CpuTray cputray;
    bool coldplug;

    private static void on_exit (int signum)
    {
        uninit_notification();
        Logger.Close();
        mainloop.quit();
    }

    public static int main (string[] args)
    {
        /* Init Log system */
        if (Logger.Init("uevt.log") == false)
            return 1;

        /* Initialise getttext */
        Intl.textdomain(GETTEXT_PACKAGE);
        Intl.bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
        Intl.bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");

        /* Load Configuration */
        config = new Config();
        config.load();

        /* Initialize Gtk */
        Gtk.init(ref args);
        Gtk.IconTheme.get_default().append_search_path(PKGDATADIR + "/icons");

        if(args[1] == "--config") {
            Configurator configurator = new Configurator();
            configurator.destroy.connect(Gtk.main_quit);
            configurator.show_all();
            Gtk.main();
            return 0;
        }

        /* coldplug - do not notify nor run commands */
        coldplug = true;

        /* Loading plugins */
        try {
            if(config.get_boolean(config.UEVT_GROUP_NOTIFY, config.UEVT_NOTIFY_POWER) ||
            config.get_boolean(config.UEVT_GROUP_NOTIFY, config.UEVT_NOTIFY_DISKS))
                init_notification();

            bool is_diskstray = config.get_boolean(config.UEVT_GROUP_DISKS,
                                                config.UEVT_DISKS_SYSTRAY);
            bool is_powertray = config.get_boolean(config.UEVT_GROUP_POWER,
                                                config.UEVT_POWER_SYSTRAY);
            bool is_cputray = config.get_boolean(config.UEVT_GROUP_CPUFREQ,
                                                config.UEVT_CPUFREQ_SYSTRAY);

            if(is_diskstray)
                diskstray = new UDisks.DisksTray();
            if(is_powertray)
                powertray = new UPower.PowerTray();
            if(is_cputray)
                cputray = new UCpu.CpuTray();

            if(config.get_boolean(config.UEVT_GROUP_DISKS, config.UEVT_UDISKS))
                UDisks.load_udisks_plugin();

            if(config.get_boolean(config.UEVT_GROUP_POWER, config.UEVT_UPOWER))
                UPower.load_upower_plugin();

        } catch(KeyFileError kf) {
           critical("Error when reading UEvt configuration : %s", kf.message);
        }

        /* Coldplug end */
        coldplug = false;

        /* Intercepting some closing signals */
        vSystem.Signal.connect(vSystem.Signal.SIGINT, on_exit);
        vSystem.Signal.connect(vSystem.Signal.SIGTERM, on_exit);

        /* Creating the Glib main loop */
        mainloop = new MainLoop (null, false);

        /* start the glib loop */
        mainloop.run();
        return 0;
    }
}