/* upower-interfaces.vala for UEvt
 *
 * Copyright (c) 2010 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace UPower {

    [DBus (name = "org.freedesktop.UPower")]
    interface Daemon: Object
    {
        public abstract async void suspend() throws IOError;
        public abstract async void hibernate() throws IOError;
        public abstract bool suspend_allowed() throws IOError;
        public abstract bool hibernate_allowed() throws IOError;
        public abstract ObjectPath[] enumerate_devices() throws IOError;

        public abstract string daemon_version { owned get; }
        public abstract bool can_suspend { owned get; }
        public abstract bool can_hibernate { owned get; }
        public abstract bool on_battery { owned get; }
        public abstract bool on_low_battery { owned get; }
        public abstract bool lid_is_present { owned get; }
        public abstract bool lid_is_closed { owned get; }

        public signal void device_added(ObjectPath device);
        public signal void device_removed(ObjectPath device);
        public signal void device_changed(ObjectPath device);
        public signal void changed();
        public signal void sleeping();
        public signal void resuming();
    }

    [DBus (name = "org.freedesktop.UPower.Device")]
    interface Device: Object
    {
        /* bug here with type and vala get_type() defined two times */
        public abstract uint Type { owned get; }
        public abstract bool power_supply { owned get; }
        public abstract bool online { owned get; }
        public abstract bool is_present { owned get; }
        public abstract uint state { owned get; }
        public abstract bool is_rechargeable { owned get; }
        public abstract double capacity { owned get; }
        public abstract int64 time_to_empty { owned get; }
        public abstract int64 time_to_full { owned get; }
        public abstract double energy { owned get; }
        public abstract double energy_empty { owned get; }
        public abstract double energy_full { owned get; }
        public abstract double energy_full_design { owned get; }
        public abstract double energy_rate { owned get; }
        public abstract double percentage { owned get; }
        public abstract double voltage { owned get; }
        public abstract uint technology { owned get; }
        public abstract string vendor { owned get; }
        public abstract string model { owned get; }
        public abstract string serial { owned get; }

        public signal void changed();
    }
}